﻿/*
 *  Watermarks.cs
 *
 * Copyright:       Copyright (C) 2021 PDF Tools AG, Switzerland
 *                  Permission to use, copy, modify, and distribute this
 *                  software and its documentation for any purpose and without
 *                  fee is hereby granted, provided that the above copyright
 *                  notice appear in all copies and that both that copyright
 *                  notice and this permission notice appear in supporting
 *                  documentation. This software is provided "as is" without
 *                  express or implied warranty.
 *
** Copyright (C) 2021 株式会社トラスト・ソフトウェア・システム
 */

/*
    ビルドするにはNuGet(nuget.org)でパッケージ「PDFTools.PdfPrinter」をインストールしてください。
    インストール手順：
        https://www.pdf-tools.trustss.co.jp/nupkg/?s=prn
*/

using System;
using Pdftools.PdfPrint;

namespace Watermarks
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                // ライセンスキー
                // 重要：以下の関数を利用するのは評価ライセンスまたはOEMライセンスの場合だけです。
                //  それ以外のライセンでは「License Manager」を使ってライセンスを有効化してください。
                if (!Printer.SetLicenseKey("適切なライセンスキー"))
                    throw new Exception("不正なライセンスキー");

                string inputPath = "input.pdf";
                string watermarkText = "承認済み";

                // PdfPrinterオブジェクト作成
                using (Printer printer = new Printer())
                {
                    string printerName = printer.DefaultPrinter;
                    if (String.IsNullOrEmpty(printerName))
                        throw new Exception(String.Format("既定のプリンターがありません。"));

                    // 入力ファイルを開く
                    if (!printer.Open(inputPath, ""))
                        throw new Exception(String.Format("入力ファイル {0} を開けません。\n" + 
                            "{1} (エラー: 0x{2:x}).", inputPath, printer.ErrorMessage, printer.ErrorCode));

                    // プリンターを開く
                    if (!printer.OpenPrinter(printerName))
                        throw new Exception(String.Format("プリンター {0} を開けません。\n" + 
                            "{1} (エラー: 0x{2:x}).", printerName, printer.ErrorMessage, printer.ErrorCode));

                    // 印刷ジョブ開始
                    if (!printer.BeginDocument("Watermark print job."))
                        throw new Exception(String.Format("プリンターに接続できません。\n" + 
                            "{0} (エラー: 0x{1:x}).", printer.ErrorMessage, printer.ErrorCode));

                    printer.Center = true;
                    printer.PaperSize = -2;

                    // 全ページを印刷
                    for (int pageNo = 1; pageNo <= printer.PageCount; pageNo++)
                    {
                        // 奇数ページの全てに「透かし」(Watermark)を追加
                        if (pageNo % 2 == 1)
                        {
                            printer.PageNo = pageNo;
                            float width = printer.PageWidth;
                            float height = printer.PageHeight;
                            float fontSize = (float)Math.Sqrt(width * width + height * height) / 
                                             watermarkText.Length;
                            printer.WatermarkBold = true;
                            printer.WatermarkOutline = true;
                            printer.WatermarkFontName = "ＭＳ 明朝";
                            printer.WatermarkFontSize = fontSize;
                            printer.WatermarkColor = 0xCCCCCC;
                            printer.WatermarkXPos = width / 5;
                            printer.WatermarkYPos = (height - height / 5 + fontSize / 2);
                            printer.WatermarkAngle = (float)Math.Atan((double)height / width);
                            printer.AddWatermarkText(watermarkText);
                        }
                        else
                        {
                            // 透かしの設定を削除
                            printer.DeleteWatermarks();
                        }

                        // ページを印刷
                        if (!printer.PrintPage(pageNo))
                            throw new Exception(String.Format("入力ファイル {1} のページ {0} をプリンター {2} に印刷できません。\n" +
                                    "{3} (エラー: 0x{4:x}).", pageNo, inputPath, printerName, printer.ErrorMessage, printer.ErrorCode));
                    }

                    // 入力ファイルを閉じる
                    printer.Close();

                    // 印刷ジョブ終了
                    if (!printer.EndDocument())
                        throw new Exception(String.Format("印刷ジョブを終了出ません。\n" +
                            "{0} (エラー: 0x{1:x}).", printer.ErrorMessage, printer.ErrorCode));
                }

                Console.WriteLine("終了");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }
    }
}
